/*  *
 * ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code Shadows.com API client implementation.
 *
 * The Initial Developer of the Original Code is
 * Pluck Corporation. This code is Copyright (C) 2005, 2006
 * All Rights Reserved.
 *
 * Contributor(s):
 *   Carl Brown (carlb@shadows.com)
 *   Duane Fields (dfields@pluck.com)
 *
 * ***** END LICENSE BLOCK ***** */


// debugging utilities

// simple profiler, call start/stop with a unique key to see elapsed time
var Profiler = {
    map: new Object(),

    start: function(key) {
        //if (! PluckDebug.debugEnabled) return;
        this.map[key] = new Date();
    },

    stop: function(key) {
        //if (! PluckDebug.debugEnabled) return;
        var stopTime = new Date();
        var startTime = this.map[key];
        if (startTime) {
            var ms = stopTime.getTime() - startTime.getTime();
            dump("Profiler: " + key + " = " + ms + " ms");
        }
    },

};

var PluckDebug = {
    debugEnabled: false,
    logFile: null,
    messageConsole: Components.classes["@mozilla.org/consoleservice;1"].
        getService(Components.interfaces.nsIConsoleService),

    traceEntry: function() {
        var r = /function (\w+)([^\{\}]*\))/;
        r.exec(dumpEntry.caller.toString());
        var f = new String(RegExp.$1);
        var args = new String(RegExp.$2);
        var msg = "Entry: " + f + args;
        this.trace(msg);
    },


    traceExit: function() {
        var r = /function (\w+)([^\{\}]*\))/;
        r.exec(dumpExit.caller.toString());
        var f = new String(RegExp.$1);
        var args = new String(RegExp.$2);
        var msg = "Entry: " + f + args;
        this.trace(msg);
    },

    // log a message to the javascript console
    trace: function(msg) {
        this.messageConsole.logStringMessage("SHADOWS.COM - " + msg);
        this.logToFile(msg);
	},

    logWarning: function(msg) {
        this.simpleLogToConsole("SHADOWS.COM - " + msg, Components.interfaces.nsIScriptError.warningFlag);
    },

    logError: function(msg) {
        this.simpleLogToConsole("SHADOWS.COM - " + msg, Components.interfaces.nsIScriptError.errorFlag);
    },

    logException: function(e, msg) {
        var mymsg;
        if (typeof e == "object") {
            mymsg = e.message;
            if (msg)
                mymsg += " (" + msg + ")";
            this.logToConsole(mymsg, e.fileName, null, e.lineNumber, null, Components.interfaces.nsIScriptError.exceptionFlag, null);
        }
        else {
            if (msg)
               mymsg = msg;
            else
               mymsg = "<no message>";
            this.simpleLogToConsole("SHADOWS.COM - " + mymsg, Components.interfaces.nsIScriptError.exceptionFlag);
        }
        if (this.debugEnabled)
           alert("Exception thrown: " + mymsg + "\n"+ this.getStackTrace(PluckDebug.logException));
    },

    // assert that fails
    fail: function(msg) {
        this.assert(false, msg);
    },

    // assert that a condition is true
    assert: function(condition, msg) {
        if (! msg) msg = "";
        if (! condition) {
            //var e = new Exception();
            if (this.debugEnabled) {
                // find out who callled us
                alert("Assertion failed: " + msg + "\n"+ this.getStackTrace(PluckDebug.assert));
            }
            this.logError("Assertion failed: " + msg);
        }
    },

    traceNode: function(node) {
		var text = shadows.Utils.xmlDocumentToText(node);
		var msg = "";
		var len = text.length;
		var start = 0;
		var end = 50;
		while (start <= len) {
			msg = msg + text.substring(start, end) + "\n";
			start = start + 50;
			end = start + 50;
			if (end > len) {
				end = len;
			}
		}
		this.trace(msg);
	},

    getStackTrace: function(source) {
        if (! source.caller) return "";
        var r = /function (\w+)([^\{\}]*\))/;
        r.exec(source.caller.toString());
        var f = new String(RegExp.$1);
        var args = new String(RegExp.$2);
        var stacktrace = f + args + "\n";
        return stacktrace + this.getStackTrace(source.caller);
    },

    simpleLogToConsole: function(aMessage, aFlags) {
        this.logToConsole(aMessage, null, null, null, null, aFlags, null);
    },
    
    logToFile: function(message) {
        try {
			if (message && message > "") {
				if (null == this.logFile) {
					this.logFile = Components.classes["@mozilla.org/file/directory_service;1"]
						.getService(Components.interfaces.nsIProperties)
						.get("ProfD", Components.interfaces.nsIFile);
					this.logFile.append("pluck.log");
				}
				if (this.logFile.exists()) {
					var fileOutputStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
					fileOutputStream.init(this.logFile, 0x10 | 0x02,	0004, null); // append mode
					message = "-------- "+(new Date().toString())+"--------\r\n"+message + "\r\n";
					fileOutputStream.write(message, message.length);
					fileOutputStream.close();
				}
			}
		}
		catch(e) {
			// do nothing at this point since we are already handling an error
		}
    },

    logToConsole: function(aMessage, aSourceName, aSourceLine, aLineNumber,
                        aColumnNumber, aFlags, aCategory) {
        var scriptError = Components.classes["@mozilla.org/scripterror;1"]
            .createInstance(Components.interfaces.nsIScriptError);
        scriptError.init(aMessage, aSourceName, aSourceLine, aLineNumber,
            aColumnNumber, aFlags, aCategory);
        this.messageConsole.logMessage(scriptError);
        this.logToFile(aMessage+" ["+aSourceName+":"+aSourceLine+":"+aLineNumber+":"+aColumnNumber+"]");
    },
};

var debugPrefs = Components.classes["@mozilla.org/preferences-service;1"].
            getService(Components.interfaces.nsIPrefBranch);
if (debugPrefs.prefHasUserValue("extensions.pluck.debug.enabled")) {
    var pref = debugPrefs.getBoolPref("extensions.pluck.debug.enabled");
    if (pref) {
        PluckDebug.debugEnabled = pref;
    }
    else {
        PluckDebug.debugEnabled = false;
    }
}

//dump("Loaded: PluckDebug.js");

