/*
     BEGIN FLOCK GPL
     
     Copyright Flock Inc. 2005-2006
     http://flock.com
     
     This file may be used under the terms of of the
     GNU General Public License Version 2 or later (the "GPL"),
     http://www.gnu.org/licenses/gpl.html
     
     Software distributed under the License is distributed on an "AS IS" basis,
     WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
     for the specific language governing rights and limitations under the
     License.
     
     END FLOCK GPL
*/

const PREF_UPLOADER_INITIAL_FILEPATH = 'flock.photo.uploader.initialPath';
const PU_INITIAL = 1;
const PU_SUCCESS = 2;
const PU_HAS_PHOTOS = 3;

var photoUploader = {};
photoUploader.SVC = Components.classes['@mozilla.org/rdf/datasource;1?name=flock-photo-uploads']
    .getService(Components.interfaces.flockIPhotoUploadService);

photoUploader.mState = PU_INITIAL;
photoUploader.mApiMgr = null;
photoUploader.mApiListener = null;

photoUploader.load = function() {
	//window.flock_photoUpload = flock_photoUpload;
    this.SVC.addListener(this.uploadListener);
    $("flock-photo-upload-slideybox").setListener(this.scrolleyboxListener);
    if(photoUploader.getPhotoCount()) {
        $("flock-photo-upload-slideybox").childNodes.item(1).selected = true;
        $("flock-photo-upload-slideybox").childNodes.item(1).focus();
    }
    this.mApiMgr = Components.classes['@flock.com/photo-api-manager;1?']
                            .getService(Components.interfaces.flockIPhotoAPIManager);
    this.mApiListener = {
        onAPIStateChange: function(aApi) {
            flock.photo.util.populateServicesPopup($('servicesMenuList').firstChild);
        },
        onError: function(aError) {
            alert(aError.errorString);
        },
    }
    
    this.mApiMgr.addListener(this.mApiListener);
    photoProperties.setupUIFields(this.mApiMgr.getDefaultService().shortName);
    this.updateCommandState();
}

photoUploader.unload = function() {
    this.SVC.removeListener(this.uploadListener);
    this.mApiMgr.removeListener(this.mApiListener);
}

photoUploader.onClose = function() {
    if (this.SVC.isUploading) {
        var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
                             getService(Components.interfaces.nsIPromptService);
        var dialogMessage = "";
        var titleMessage = "";
        
        titleMessage = "Really close Photo Uploader ?";
        dialogMessage = "Are you sure you want to close the Photo Uploader while a photo upload ";
        dialogMessage += "is in progress ?\n\nYou can always resume uploading your photos when you ";
        dialogMessage += "relaunch the Photo Uploader."
        var rv = promptService.confirm(null, titleMessage, dialogMessage);
        if (rv) {
            this.SVC.cancelUpload();
            return true;
        } else {
            return false;
        }
    } else
        return true;

}

photoUploader.cancelUpload = function() {
    this.mRecentlyCancelled = true;
    this.SVC.cancelUpload();
}

photoUploader.updateCommandState = function(aDisable) {
    //first, see if we have uploads
    if(photoUploader.SVC.isUploading || aDisable) {
        photoUploader.disableSlideybox();
        photoProperties.clearSelection();
        $("flock-photo-upload-actions").setAttribute("hidden", true);
        $("flock-photo-cancel-actions").removeAttribute("hidden");
        $("cmd_removePhoto").setAttribute("disabled", true);
        $("cmd_doUpload").setAttribute("disabled", true);
        $("cmd_addFiles").setAttribute("disabled", true);
        $("cmd_launchSettings").setAttribute("disabled", true);
        return;
    }
    $("flock-photo-upload-actions").removeAttribute("hidden");
    $("flock-photo-cancel-actions").setAttribute("hidden", true);

    $("cmd_addFiles").removeAttribute("disabled");
    $("cmd_launchSettings").removeAttribute("disabled");

    var count = this.getPhotoCount();
    var selectionCount = this.getSelectionCount();


    if(count==0) {
        // disable props, upload
        // setup message
        this.mState = PU_INITIAL;
        $("cmd_doUpload").setAttribute("disabled", true);
    }
    else {
        this.mState = PU_HAS_PHOTOS;
        $("cmd_doUpload").removeAttribute("disabled");
    }

    if(selectionCount==0) {
        $("cmd_removePhoto").setAttribute("disabled", true);
    }
    else {
        $("cmd_removePhoto").removeAttribute("disabled");
    }
    this.updateMessage();
}

photoUploader.updateMessage = function(aOverride) {
    if(aOverride) {
        this.mState = aOverride;
    }
    var message = "";
    switch(this.mState) {
        case PU_INITIAL:
            $("flock-photo-instructions").removeAttribute("hidden");
            message = $("flock-photo-instructions-message").getAttribute("msgInitial");
        break;
        case PU_SUCCESS:
            $("flock-photo-instructions").removeAttribute("hidden");
            message = $("flock-photo-instructions-message").getAttribute("msgSuccess");
        break;
        case PU_HAS_PHOTOS:
            $("flock-photo-instructions").setAttribute("hidden", true);
            message = "";
        break;
    }
    dump(message + "<<<<<<MESSAGE\n");
    $("flock-photo-instructions-message").value = message;
}

photoUploader.getPhotoCount = function() {
    return ($("flock-photo-upload-slideybox").childNodes.length - 1);
}

photoUploader.getSelectionCount = function() {
    var selection = $("flock-photo-upload-slideybox").getSelection();
    return selection.length;
}

photoUploader.getAllPhotoUploads = function() {
    var photouploads = document.getElementsByTagName('photoupload');
    var uploadsArray = Array();
    for (var i=1; i < photouploads.length; i++) {
        uploadsArray.push(photouploads[i]);
    }
    return uploadsArray;
}

photoUploader.addPhotoFiles = function() {
    const nsIFilePicker = Components.interfaces.nsIFilePicker;
    var init_dir = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
    var init_path_pref = flock_getCharPref(PREF_UPLOADER_INITIAL_FILEPATH);
    var photoUrls = Array();

    if (!init_path_pref) {
        var dirService = Components.classes["@mozilla.org/file/directory_service;1"]
                 .getService(Components.interfaces.nsIProperties);
                 
// platform specificness                 
//@line 191 "/home/build/tinderbuild/src/flock/mozilla/browser/components/flock/photo/content/photoUploader.js"
        init_path_pref = dirService.get("Home", Components.interfaces.nsIFile).path;
//@line 193 "/home/build/tinderbuild/src/flock/mozilla/browser/components/flock/photo/content/photoUploader.js"

        init_dir.initWithPath(init_path_pref);
        
        if (init_dir.path && init_dir.path.length)
            init_path_pref = init_dir.path;
        
        flock_setCharPref(PREF_UPLOADER_INITIAL_FILEPATH, init_path_pref);
        
    } else {
        init_dir.initWithPath(init_path_pref);
    }

    
    
    var fp = Components.classes["@mozilla.org/filepicker;1"]
    	     .createInstance(nsIFilePicker);
    	
    fp.init(window, "Add photos", nsIFilePicker.modeOpenMultiple);
    fp.appendFilters(nsIFilePicker.filterAll);
  
    var homeDir = null;
    var mypicsPath = null;

    fp.displayDirectory = init_dir;
      
    if ( fp.show( ) == nsIFilePicker.returnOK && fp.files ) {
        var enum = fp.files;
        var item = null;
        var file = null;
        var path = null;  
        while (enum.hasMoreElements()) {
            item = enum.getNext();
            file = item.QueryInterface(Components.interfaces.nsIFile);
            path = file.path.toLowerCase();
            if (path.match(/.+jpg$/) ||
                 path.match(/.+jpeg$/)||
                 path.toLowerCase().match(/.+png$/) ||
                 path.toLowerCase().match(/.+gif$/)) 
                {
                    //flock.photo.util.handlePhotoDrop(flock_getURLFromFile(file), null, false);   
                    photoUrls.push(flock_getURLFromFile(file));
               }
       }       
        //flock.photo.util.processPhotoImportQueue();
        var dropProcessor = new PhotoDropProcessor(photoUrls, null); 
        dropProcessor.start();
        var lastSelectedDir  = file.path.substring(0, file.path.indexOf(file.leafName));
        flock_setCharPref(PREF_UPLOADER_INITIAL_FILEPATH, lastSelectedDir);
        $('flock-photo-instructions').setAttribute('hidden', true);
    }
}
       

photoUploader.enableSlideybox = function() {
    $("flock-photo-disable").setAttribute("hidden", true);
}

photoUploader.disableSlideybox = function() {
    $("flock-photo-disable").removeAttribute("hidden");
}

// this may move to slideybox
photoUploader.selectFirstPhoto = function() {
    $("flock-photo-upload-slideybox").firstChild.nextSibling.selected = true;
    $("flock-photo-upload-slideybox").firstChild.nextSibling.focus();
}

photoUploader.uploadListener = {
    mErrorOnUpload: false,
    mWindow: window,
    onUploadStart: function (aCurrentUpload, aCount, aMax) {
        // clear properties selection
        photoUploader.mRecentlyCancelled = false;
        photoUploader.uploadListener.mErrorOnUpload = false;
        photoUploader.updateCommandState();
        var msg = "Uploading Photo " + aCount + " of " + aMax + ": " + aCurrentUpload.title;
       // -1 because the current photo is not uploaded yet
        var percent = ((aCount - 1)*100) / aMax;
        $('uploading-title').setAttribute("value", msg);
        $('upload-progressmeter').setAttribute("value", percent.toString()+"%");
    },
    onUpload: function (aCurrentUpload, aCount, aMax) {
    },
    onUploadAdd: function (aCurrentUpload) {
        if(!aCurrentUpload) {
            photoProperties.synch();
            photoProperties.invalidate();
        }
        else {
            if(photoProperties.updateSelectedItem(aCurrentUpload)) {
                $(aCurrentUpload.id).parentNode.selected = true;
            }
            photoUploader.updateCommandState();
            if(photoUploader.getPhotoCount()==1) {
                photoUploader.selectFirstPhoto();
            }
        }
    },
    onUploadRemove: function () {
        if(photoUploader.SVC.isUploading) return;
        photoUploader.updateCommandState();
    },
    onUploadComplete: function () {
        $('flock-photo-upload-deck').selectedIndex=0;
        photoUploader.enableSlideybox();
        photoUploader.updateCommandState();
        // Delete the temporary directory, unless the single photo uploader is open
        
        if (!this.mErrorOnUpload && !photoUploader.mRecentlyCancelled)
        {
            var window = this.mWindow;
            if (!singleUploadWindow)
              window.setTimeout("photoUploader.SVC.cleanPhotoUploadCache();", 1000);
            var showUploadedPics = flock_getBoolPref('flock.photo.uploader.showPhotosOnUpload');
            if (showUploadedPics) {
                var apimgr = Components.classes['@flock.com/photo-api-manager;1?'] 
                                  .getService(Components.interfaces.flockIPhotoAPIManager);
                var authPerson = apimgr.getDefaultService().getAuthPerson();
                var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                          .getService(Components.interfaces.nsIWindowMediator);
                var win = wm.getMostRecentWindow('navigator:browser');
                
                if (win) {
                    win.flock_topbarSelect(win.document.getElementById('flock_PhotoBrowseTopbarBroadcaster'), true);
                    if (win.TopbarJSCtx().photoBrowser) win.TopbarJSCtx().photoBrowser.loadPersonPre(authPerson);
                }
            }
            var windowMediator = Components.classes["@mozilla.org/appshell/window-mediator;1"]
              .getService(Components.interfaces.nsIWindowMediator);
            var singleUploadWindow = false;
            var allWindows = windowMediator.getEnumerator(null);
            while(allWindows.hasMoreElements()) {
              var window = allWindows.getNext();
              if (window.name == "singlePhotoUploading")
                singleUploadWindow = true;
            }
            window.setTimeout("photoUploader.updateMessage(PU_SUCCESS);", 0);//force override
        }
    },
    onUploadError: function (aUpload, aCount, aMax, aError) {
        var errorMessage = aError.errorString;
        // if it's an unknow error display any addition info the server might return
        if (aError.errorCode == aError.PHOTOSERVICE_UNKNOWN_ERROR)
            errorMessage += "\n\nAdditionally, the server returned the message " + aError.serviceErrorString;
        alert(aError.errorString);
        photoUploader.enableSlideybox();
        this.mErrorOnUpload = true;
    },
    onAuthStateChange: function (aState) {
    },
    onUploadCreateError: function(aFile) {
        alert("The photo " + aFile + " is invalid and cannot be added to the photo uploader.");
        photoUploader.updateCommandState();
    },
}

photoUploader.scrolleyboxListener = {
    onDrop: function(aHoverChild, aSelection) {
        try {
            photoProperties.synch();
            var svc = photoUploader.SVC;
            var hoverChild = aHoverChild;
            if(hoverChild) hoverChild = svc.getPhotoUpload(hoverChild.firstChild.id);
            var ids = new Array();
            for(var i=0;i<aSelection.length;++i) {
                ids[i] = aSelection[i].firstChild.id;
            }
            var hoverID = null;
            if(aHoverChild) hoverID = aHoverChild.firstChild.id;

            for(var i=0;i<ids.length;++i) {
                if(hoverID && hoverID==ids[i]) continue;
                svc.addPhotoUpload(hoverChild, svc.getPhotoUpload(ids[i]));
            }
            for(var i=0;i<ids.length;++i) {
                $(ids[i]).parentNode.selected = true;
            }
        }
        catch(e) {
            alert(e);
        }
    },
    onScrollRight: function() {
    },
    onScrollLeft: function() {
    },
    onSelect: function(aSelection) {
        if(photoUploader.SVC.isUploading) return;
        photoUploader.updateCommandState();
        var uploadItems = Array();
        for (var i=0; i < aSelection.length; i++)
            uploadItems.push(aSelection[i].firstChild)
        
         photoProperties.setSelection(uploadItems);
        // do the selection
    },
    onRemoveSelection: function(aSelection) {
        // do the clear selection

        //remove from service
        var svc = photoUploader.SVC;
        photoUploader.removeSelection(aSelection);
        
        
        //update state
    }
}

photoUploader.doUpload = function() {
    var params = {};
    params.photoUploads = this.getAllPhotoUploads();
    var rv = flock.photo.util.launchUploadOptions(params);

    if (params.launchUploader) {
        $('flock-photo-upload-deck').selectedIndex=1;
/*        var progressParams = {
            apiName: params.apiName,
            albumId: params.albumId,
            photoUploader: this
        };
        window.openDialog("chrome://browser/content/flock/photo/photoUploadProgress.xul", "_blank", "modal,centerscreen", progressParams);*/
        /* if (params.viewService) {
            window.opener.open(photoServiceUrl, "CtrlWindow", "toolbar=yes,menubar=yes,location=yes,scrollbars=yes,resizable=yes");
        } */
    }
    // set the album id on the photo uploads
    // this should probably be an attribute on the service
    // not each particular photo upload object
    var uploads = this.getAllPhotoUploads();
    for (var i=0; i < uploads.length; i++) {
        uploads[i].album = params.albumId;
        this.SVC.savePhotoUpload(uploads[i]);
    }
    
    if (params.apiName) {
        window.setTimeout(this.reallyDoUpload, 0, params.apiName);
        this.updateCommandState(true);
//        this.reallyDoUpload(params.apiName);
    }
}

photoUploader.reallyDoUpload = function(aApiShortname) {

    var api = photoUploader.mApiMgr.getAPIFromShortname(aApiShortname);
    photoUploader.mApiMgr.setDefaultService(aApiShortname);
    photoUploader.SVC.upload(api);
}

photoUploader.removeSelection = function(aSelection) {
    if (!aSelection) {
        aSelection = $("flock-photo-upload-slideybox").getSelection();
    }
    
    photoProperties.clearSelection();
    for (var i=0; i < aSelection.length; i++) {
        this.SVC.removePhotoUpload(aSelection[i].firstChild);
    }
    
}

photoUploader.startUploadFlow = function() {
    var inst = this;
    var service = this.mApiMgr.getDefaultService();
    if (service) {
        if(service.isLoggedIn)
            this.doUpload();
        else {
            var loginListener = {
                onAuth: function() {
                    if (service.isLoggedIn) {
                        inst.doUpload();
                    } 
                },
                
                onError: function(aError) {
                    alert(aError.errorString);
                },
            }
           var message =  "Flock is opening " + service.serviceName + "'s Sign In page. Follow " + 
                            service.serviceName + "'s instructions to sign in.";
           message += "\n";
           message += "Be sure to close the " + service.serviceName  + " page when you are signed in.";
           alert(message);
           
           // there is a bug in the apimanager which causes multiple state changes
           // which prevents us to use it -- for now call the login on the photoservice directly
           service.login(loginListener);
      }
    } else {
        var params = {};
        params.hasValidSvc = false;
        rv = flock.photo.util.launchSettings(params);
        if (params.hasValidSvc)
            this.doUpload();
    }
}
